require "watir"
require "watir-webdriver"

module Symbiote
  include Situation

  module_function

  def elements
    @elements = Watir::Container.instance_methods unless @elements
  end

  def hosts?(method)
    Symbiote.elements.include? method.to_sym
  end

  module Elements
    module Accessor
      def reference_element(element, locator)
        browser.send(element, *locator).to_subtype
      rescue NoMethodError
        browser.send(element, *locator)
      end
    end

    def method_missing(method, *args)
      abort("The method '#{method}' is not a recognized element method.")
      super
    end

    def respond_to_missing?(method)
      super
    end

    Symbiote.elements.each do |element|
      define_method(element) do |*signature, &block|
        identifier, locator = parse_signature(signature)
        define_element_accessor(identifier, locator, element, &block)
      end
    end

    private

    def define_element_accessor(identifier, *locator, element, &block)
      define_method(identifier.to_s.to_sym) do |*values|
        if block_given?
          instance_exec(*values, &block)
        else
          no_locator(self.class, identifier) if empty_locator(locator, values)
          locator = values if locator[0].nil?
          reference_element(element, locator)
        end
      end
    end

    def parse_signature(signature)
      [signature.shift, signature.shift]
    end
  end
end
